<template>
    <div>
        <el-row
            type="flex"
            ref="clientCardRef"
            class="rowStyle cursor--pointer"
            justify="space-between"
            :class="{'new': newAppointment }"
            @click.native="viewDetail"
        >
        <div class="test">
            <div class="unread" v-if="isReserved">
                <div class="circle" v-if="!isRead"></div>
            </div>

            <div class="pl-10">
                <div class="smTxt_bold name">{{ client.name }}</div>
                <div class="my-10 xsTxt">預約成功</div>
                <div class="professionals">
                    <template v-if="client.requirement">
                        <span
                            v-for="(item, index) in requirements"
                            :key="index"
                            class="professionalsTxt"
                        >#{{item}}</span>
                    </template>
                    <template v-else>
                        <span class="professionalsTxt noProfessionalsTxt"
                        >(客戶未提供需求項目)</span>
                    </template>
                </div>
                <AppointmentProgress
                  :currentStep="client.communicateStatus"
                ></AppointmentProgress>
            </div>
        </div>
            <!-- <el-col :xs="5" :sm="3" align="middle">
                <el-avatar
                    :size="50"
                ></el-avatar>
                <div class="satisfaction" v-if="!hideReviews">
                    <template v-if="client.satisfactionScore">
                        TODO:隱藏滿意度
                        <i class="icon-star pam-icon icon--yellow satisfaction"></i>
                        <span>{{client.satisfactionScore}}</span>
                    </template>
                    <template v-else>
                        <div class="unfilled">未填滿意度</div>
                    </template>
                </div>
            </el-col> -->

            <div class="flex-column contactInfo" :xs="4" :sm="6">
                <!-- <div
                    class="smTxt_bold fix-chrome-click--issue"
                    :class="{'unread-txt': reservedTxt === '未讀', 'read-txt': reservedTxt !== '未讀'}"
                >{{ reservedTxt }}</div> -->
                <div class="invite-msg smTxt_bold">
                    傳送約訪通知
                </div>
                <div
                    class="date xsTxt text--black"
                >{{ date }}</div>
                <div
                    class="xsTxt text--mid_grey"
                >{{ time }}</div>
            </div>
        </el-row>

        <Ui-Dialog
            :isVisible.sync="isVisibleDialog"
            :width="dialogWidth"
            @closeDialog="closeDialog"
            class="pam-myDemand-dialog"
        >
            <h5 class="subTitle text--center mb-30"
            >{{isReserved ? '預約資訊' : '已聯絡資訊'}}</h5>

            <p v-if='isReserved'
                class="smTxt text--right"
            ><span v-if="isRead">{{client.consultantReadTime | formatDate}}</span> 已讀</p>
            <p
                v-if="!isReserved"
                class="smTxt text--right"
            >{{client.contactTime | formatDate}} 聯絡</p>
            <p class="smTxt">{{client.appointmentDate | formatDate}} 預約</p>

            <div class="dialogTxt">
                <p>姓名：<span>{{client.name}}</span></p>
                <p>電話：<span>{{client.phone}}</span></p>
                <p>Email：<span>{{client.email}}</span></p>
                <p>性別：<span>{{gender}}</span></p>
                <p>年齡：<span>{{client.age | toAgeLabel }}</span></p>
                <p>職業：<span>{{client.job}}</span></p>
                <p>需求：<span>{{client.requirement.split(',').join('、')}}</span></p>
                <p v-for="(item, index) in hopeContactTime"
                    :key="index"
                >連絡時段{{index + 1 | formatNumber}}：<span>{{ item | formatHopeContactTime}}</span></p>
            </div>
            <div class="mt-30">
                <div class="memoTitleStyle">
                    <div class="mdTxt">內容描述</div>
                    <div
                        class="smTxt text--bold text--primary cursor--pointer text--underline edit"
                        @click='editMemo'
                    >編輯</div>
                </div>

                <el-input
                    class="mt-10 pam-appointment-textarea"
                    type="textarea"
                    :rows="3"
                    maxlength="100"
                    placeholder="請輸入，限100字。"
                    :disabled="!isEdit"
                    v-model="memo"
                >
                </el-input>

                <div class="mt-10 smTxt text--bold text--primary text--right fixed-Height">
                    <template v-if="isEdit">
                        <span class="cursor--pointer" @click="cancelEditMemo">取消</span>
                        <span class="pl-20 cursor--pointer" @click="saveMemo">儲存</span>
                    </template>
                </div>
            </div>
            <div class="mt-30 text--center" v-if="isReserved">
                <el-button @click="markAppointment">標註為已連絡</el-button>
            </div>
        </Ui-Dialog>
    </div>
</template>

<script lang="ts">
import { Vue, Component, Prop, Action, namespace, Watch } from 'nuxt-property-decorator';

import appointmentService from '~/shared/services/appointment.service';
import UtilsService from '~/shared/services/utils.service';
import { hideReviews } from '~/shared/const/hide-reviews';
import myConsultantService from '~/shared/services/my-consultant.service';
import { ElRow } from 'element-ui/types/row';
import { Appointment, AppointmentMemoInfo } from '~/shared/models/appointment.model';

const localStorage = namespace('localStorage');
@Component({
    filters: {
        formatNumber(index: number) {
            if (index) {
                const upperNumber = ['零', '一', '二', '三', '四', '五', '六', '七', '八', '九', '十']
                return upperNumber[index];
            }
        },
        formatHopeContactTime(item: string): string {
          if (item) {
            const [hopeDay, hopeTime] = item.split('、');
            const day = hopeDay.split(',').length > 6 ? '不限日期' : hopeDay;
            const time = hopeTime.split(',').length > 3 ? '不限時間' : hopeTime;
            return `${day}、${time}`;
          }
          return '';
        }
    }
})
export default class ClientList extends Vue {
    @Action
    updateMyAppointment!: (data: Appointment) => void;

    @Prop()
    client!: Appointment;

    @localStorage.Mutation
    storageClearAppointmentIdFromMsg!: () => void;

    isVisibleDialog = false;
    dialogWidth = '';
    hideReviews = hideReviews;

    isEdit = false;
    memoInfo: AppointmentMemoInfo = {
        appointmentId: 0,
        content: '',
        id: 0
    }
    memo = '';

    //////////////////////////////////////////////////////////////////////

    @Watch('$route', {immediate: true})
    onRouteChange() {
        const appointmentIdFromMsg = this.$route.query.appointmentId;
        if (appointmentIdFromMsg && +appointmentIdFromMsg === this.client.id) {
            this.openDetail();
        }
    }

    //////////////////////////////////////////////////////////////////////

    mounted() {
        this.memoInfo = this.client.appointmentMemoList.length > 0
            ? JSON.parse(JSON.stringify(this.client.appointmentMemoList[0]))
            : {appointmentId: 0, content: '', id: 0};
        this.memo = this.memoInfo.content;
    }

    //////////////////////////////////////////////////////////////////////

    viewDetail(): void {
      this.$router.push(`/appointment/${this.client.id}`);
    }

    get newAppointment(): boolean {
      return !this.client.consultantViewTime
            && this.client.communicateStatus === 'reserved';
    }

    get isReserved() {
      return this.client.communicateStatus === 'reserved';
    }

    get isRead() {
      return !!this.client.consultantReadTime;
    }

    get requirements() {
        return this.client.requirement.split(',');
    }

    get gender() {
        if (this.client.gender) {
            return this.client.gender === 'male' ? '男性' : '女性';
        }
        return '';
    }

    get hopeContactTime() {
        const contactList = this.client.hopeContactTime.split("'").map(item => item.slice(0, item.length));
        return contactList.filter(item => !!item && item !== ",")
    }

    get reservedTxt(): string {
        if (this.isReserved) {
            return this.client.consultantReadTime ? '已讀' : '未讀';
        } else {
            return '已聯絡'
        }
    }

    get displayTime(): string {
        if (this.isReserved) {
            return this.client.appointmentDate;
        } else {
            return this.client.lastModifiedDate;
        }
    }

    get time() {
        const formatDate = (this.$options.filters as any).formatDate(this.displayTime);
        return formatDate.split(' ')[1]
    }

    get date() {
        const formatDate = (this.$options.filters as any).formatDate(this.displayTime);
        return formatDate.split(' ')[0];
    }

    openDetail() {
        setTimeout(() => {
            (this.$refs.clientCardRef as any).$el.classList.add('currentShowStyle');
        }, 0)
        this.dialogWidth = UtilsService.isMobileDevice() ? '80%' : '';
        this.isVisibleDialog = true;
    }

    markAppointment() {
        myConsultantService.markAsContact(this.client.id).then(data => {
            this.updateMyAppointment(data);
            this.isVisibleDialog = false;
        })
    }

    closeDialog(): void {
      const unread = !this.client.consultantReadTime;
        if (unread) {
            appointmentService.recordRead(this.client.id).then((_) => {
                const updatedClient = {...this.client};
                updatedClient.consultantReadTime = new Date().toString();
                this.updateMyAppointment(updatedClient);
            });
        };
        this.isEdit = false;
        this.clearAppointmentIdFromMsg();
    }

    private clearAppointmentIdFromMsg() {
        this.storageClearAppointmentIdFromMsg();
        this.$router.push({query: {}});
        setTimeout(() => {
            (this.$refs.clientCardRef as ElRow).$el.classList.remove('currentShowStyle')
        },1000)
    }

    saveMemo() {
        if (this.client.appointmentMemoList.length > 0) {
            const params = {
                content: this.memo,
                id: this.client.appointmentMemoList[0].id
            };
            this.updateMemo(params);
            return;
        }

        const params = {
            content: this.memo,
            appointmentId: this.client.id,
        }
        this.createMemo(params);
    }

    private createMemo(params) {
        appointmentService.createMemo(params).then(memoRes => {
            this.storeMemo(memoRes);
        });
    }

    private updateMemo(params) {
        appointmentService.updateMemo(params).then(memoRes => {
            this.storeMemo(memoRes);
        });
    }

    private storeMemo(memoRes) {
        this.memoInfo = memoRes;
        this.memo = this.memoInfo.content;
        this.client.appointmentMemoList[0] = this.memoInfo;
        this.isEdit = false;
    }

    editMemo() {
        this.isEdit = !this.isEdit;
        this.memo = this.memoInfo.content;
    }

    cancelEditMemo() {
        this.isEdit = false;
        this.memo = this.memoInfo.content;
    }

}
</script>

<style lang="scss" scoped>
    .rowStyle {
        padding: 10px 15px 10px 5px;
        background-color: $PRIMARY_WHITE;
        margin-bottom: 10px;
        display: flex;
        justify-content: space-between;

        transition: background-color 0.5s;
        &.new {
            border-left: solid 4px $YELLOW;
        }
        &.currentShowStyle {
            background-color: rgba(236, 195, 178, 0.5);
            transition: background-color 0.5s;
        }
        .unread {
            align-self: center;
            .circle {
                width: 10px;
                height: 10px;
                border-radius: 50%;
                background-color: $PRIMARY_RED;
                margin: auto;
            }
        }
        .satisfaction {
            font-size: 12px;
            font-weight: bold;
            margin-top: 5px;
            .unfilled {
                font-weight: lighter;
                color: $MID_GREY;
            }
        }
        .professionals {
            overflow: hidden;
            white-space: nowrap;
            text-overflow: ellipsis;
            .professionalsTxt {
                font-size: 12px;
                margin-right: 5px;
            }
            .noProfessionalsTxt {
                color: $PRUDENTIAL_GREY;
                font-weight: lighter;
            }
        }
        .contactInfo {
            text-align: right;
            .date {
                font-size: 12px;
            }
        }
        .unread-txt {
            @extend .text--primary;
        }
        .read-txt {
            color: $SKY_BLUE;
        }
    }
    .flex-column {
        display: flex;
        flex-direction: column;
        justify-content: space-between;
    }
    .dialogTxt {
        font-size: 20px;
        overflow-y:scroll;
        max-height: 25vh;
        @include desktop {
            height: 400px;
        }
    }
    .memoTitleStyle {
        display: flex;
        flex-direction: row;
        justify-content: space-between;

        .edit {
            align-self: flex-end;
        }
    }
    .fixed-Height {
        height: 16px;
    }
    .test{
        display: flex;
    }
    .invite-msg{
        color: #ED1B2E;
        border-bottom: 1px solid #ED1B2E;
        width: 97px;
    }
    .line{
        height: 4px;
        width: 10px;
        background-color: $PRUDENTIAL_GREY;

    }
</style>
